function nodes = BNfromAdjMat(mat, discrete, nodenames)
%nodes = BNfromAdjMat(mat, discrete, nodenames)
%
% Takes as input an adjacency matrix and output the array of NODE structures
% that corresponds to a hybrid discrete/continuous Bayesian Network with
% the same connectivity.
% Maintains order of nodes according to listing in MAT.
%
% INPUT : 
%   MAT : adjacency matrix, such that if MAT(i,j) is non-zero, there is a
%       link from NODES(i) to NODES(j)
%   DISCRETE : boolean array indicating which nodes are discrete
%   NODENAMES : optional names for each of the nodes
%
% OUTPUT : 
%   NODES : array of NODE structures that encodes the Bayes Net.
%
%
% Copyright Michael McGeachie, 2010.  MIT license. See cgbayesnets_license.txt.

if (nargin < 2)
    discrete = true(1, size(mat,1));
end

if (nargin < 3)
    nodenames = cell(size(discrete));
    for i=1:length(discrete)
        nodenames{i} = num2str(i);
    end
end

nodes = repmat(Node(),1,size(mat,1));

for i = 1:length(nodes)
    nodes(i).self = nodenames{i};
    nodes(i).discrete = discrete(i);
end

for i = 1:length(nodes)
    for j = 1:length(nodes)
        if (mat(i,j) ~= 0)
            if (isempty(nodes(i).children))
                nodes(i).children = {nodes(j).self};
                nodes(i).cindex = j;
            else
                nodes(i).children{end+1} = nodes(j).self;
                nodes(i).cindex(end+1) = j;
            end
            
            if (isempty(nodes(j).parents))
                nodes(j).parents = {nodes(i).self};
                nodes(j).pindex = i;
             else
                nodes(j).parents = {nodes(j).parents{:},nodes(i).self};
                nodes(j).pindex(end+1) = i;
            end
        end
    end
end

